package	jp.co.sds_corp.ratemixer.fx;

import jp.co.sds_corp.fxutil.CcyPair;
import jp.co.sds_corp.transfer.IDataKey;

/**
* 為替レートのキー情報を保持するクラスです。
*/
public class FxRateKey implements IDataKey
{
	private static final long serialVersionUID = 1L;
	CcyPair		pair;
	String		term;

	/**
	* 指定された通貨ペアと期間で、レート情報を構築します。
	* @param	pair	通貨ペア
	* @param	term	期間
	*/
	public FxRateKey( CcyPair pair, String term ) {
		this.pair = pair;
		this.term = term;
	}

	/**
	* 引数で指定されたオブジェクトとレートを比較します。
	* @param	cmp		比較元のレート情報
	* @return	比較結果。変更無し:true 変更有り:false
	*/
	public boolean match( IDataKey cmp ) {
		FxRateKey fx = (FxRateKey )cmp;
		if ( pair != null && !pair.equals( fx.pair )) {
			return false;
		}
		if ( term != null && !term.equals( fx.term )) {
			return false;
		}
		return true;
	}

	/**
	* 引数で指定されたオブジェクトとレートを比較します。
	* @param	o		比較元のレート情報
	* @return	比較結果。変更無し:true 変更有り:false
	*/
	public boolean equals( Object o ) {
		if ( !(o instanceof IDataKey )) {
			return false;
		}
		IDataKey cmp = (IDataKey )o;
		FxRateKey fx = (FxRateKey )cmp;
		if ( pair == null ) {
			if ( fx.pair != null ) {
				return false;
			}
		}
		if ( term == null ) {
			if ( fx.term != null ) {
				return false;
			}
		}
		if ( pair != null && !pair.equals( fx.pair )) {
			return false;
		}
		if ( term != null && !term.equals( fx.term )) {
			return false;
		}
		return true;
	}

	/**
	* オブジェクトのハッシュコード値を返します。
	* @return	ハッシュコード値
	*/
	public int hashCode() {
		int ret = pair.getCcy1().hashCode() + pair.getCcy2().hashCode()
				+ term.hashCode();
		return ret;
	}

	/**
	* レートの文字列表現を返します。
	* @return	レートの文字列表現
	*/
	public String toString() {
		return pair +":"+ term;
	}

	/**
	* 通貨ペア(pair)を取得します。
	* @return	pairの内容
	*/
	public CcyPair getCcyPair() {
		return pair;
	}

	/**
	* 期間コード(term)を取得します。
	* @return	termの内容
	*/
	public String getTerm() {
		return term;
	}

}
