package	jp.co.sds_corp.ratemixer.fx.command;

import java.net.MalformedURLException;
import java.rmi.Naming;
import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;
import java.rmi.server.UnicastRemoteObject;
import java.util.Map;

import jp.co.sds_corp.fxutil.Ccy;
import jp.co.sds_corp.fxutil.CcyPair;
import jp.co.sds_corp.fxutil.ForexRate;
import jp.co.sds_corp.fxutil.IRateLoader;
import jp.co.sds_corp.fxutil.MasterCache;
import jp.co.sds_corp.ratemixer.fx.FxRate;
import jp.co.sds_corp.util.config.IConfiguration;
import jp.co.sds_corp.util.daemon.AbstractService;
import jp.co.sds_corp.util.log.CmnLogFactory;
import jp.co.sds_corp.util.log.CmnLogger;

public class RMIServer extends AbstractService implements ICommandServer
{
	private static final long serialVersionUID = 1L;
	CmnLogger logger = CmnLogFactory.getInstance().getLogger(this);
	String host;
	int port;
	String name;
	String url;
	IRateLoader	loader;
	Receiver rcv;

	/**
	* 指定された初期情報でDummySourceを初期化します。
	* @param    config  初期情報
	*/
	public void init( IConfiguration config ) throws Exception {
		super.init( config );

		rcv = new Receiver();
		host = config.get( "host" );
		port = Integer.valueOf( config.get( "port" ) ).intValue();
		name = config.get( "name" );

		Registry    reg;
		try {
			reg = LocateRegistry.createRegistry( port );
		} catch ( RemoteException ex ) {
			logger.debug( "getRegistry:" + port + "(" + ex + ")" );
			// Registry起動済み
			reg = LocateRegistry.getRegistry( port );
		}
		url = "rmi://" + host + ":" + port + "/" + name;
		logger.debug( "bind:" + url );
		Naming.bind( url, rcv );
	}

	/**
	* このサービスを開始します。
	*/
	public void start() {
	}

	/**
	* このサービスを停止します。
	*/
	public void stop() {
		try {
			Naming.unbind( url );
		} catch ( Exception ex ) {
			logger.warn( "unbind error", ex );
		}
	}

	public void setRateLoader( IRateLoader loader ){
		this.loader = loader;
	}

	public class Receiver extends UnicastRemoteObject implements IRMIServer
	{
		private static final long serialVersionUID = 1L;
		MasterCache	master = MasterCache.getInstance();
		public Receiver() throws RemoteException, MalformedURLException {
		}

		public Map<String,Ccy> mapCcy() throws Exception {
			return master.mapCcy();
		}

		public Map<String,CcyPair> mapCcyPair() throws Exception {
			return master.mapCcyPair();
		}

		public FxRate getSpotRate( String ccy1, String ccy2 )
			throws Exception
		{
			if ( loader == null ) {
				throw( new CommandException( "loader not set" ));
			}
			CcyPair pair = master.getCcyPair( ccy1, ccy2 );
			if ( pair == null ) {
				throw( new CommandException( "ccypair[" + ccy1 + "/" + ccy2
					+ "] not found." ));
			}
			FxRate rate = new FxRate( pair, "SP" );
			ForexRate fxr = loader.getSpotRate( pair );
			if ( fxr == null ) {
				throw( new CommandException( "spot rate[" + ccy1 + "/" + ccy2
					+ "] not found." ));
			}
			rate.copyFrom( fxr );
			return rate;
		}
		public CcyPairProperty getCcyPairProperty( String ccy1, String ccy2 )
			throws Exception
		{
			CcyPair pair = master.getCcyPair( ccy1, ccy2 );
			if ( pair == null ) {
				return null;
			}
			CcyPairProperty ret = new CcyPairProperty();
			String var;
			var = pair.getProperty( "Spread" );
			if ( var != null ) {
				ret.setSpread( Integer.valueOf( var ).intValue() );
			}
			var = pair.getProperty( "Skew" );
			if ( var != null ) {
				ret.setSkew( Integer.valueOf( var ).intValue() );
			}
			var = pair.getProperty( "Bias" );
			if ( var != null ) {
				ret.setBias( Integer.valueOf( var ).intValue() );
			}
			return ret;
		}

		public void setCcyPairProperty( String ccy1, String ccy2,
				CcyPairProperty prop )
			throws Exception
		{
			CcyPair pair = master.getCcyPair( ccy1, ccy2 );
			if ( pair == null ) {
				CommandException cex = new CommandException(
						"CcyPair not found.[" + ccy1 + "/" + ccy2 + "]" );
				throw( cex );
			}
			synchronized ( pair ) {
				pair.setProperty( "Spread", String.valueOf( prop.getSpread() ));
				pair.setProperty( "Skew", String.valueOf( prop.getSkew() ));
				pair.setProperty( "Bias", String.valueOf( prop.getBias() ));
			}
		}
		public void setStatus( String ccy1, String ccy2, int status )
			throws Exception
		{
			CcyPair pair = master.getCcyPair( ccy1, ccy2 );
			if ( pair == null ) {
				CommandException cex = new CommandException(
							"CcyPair not found.[" + ccy1 + "/" + ccy2 + "]" );
				throw( cex );
			}
			synchronized ( pair ) {
				pair.setProperty( "Status", String.valueOf( status ));
			}
		}

	}

}
