package	jp.co.sds_corp.ratemixer.fx.filter;

import java.util.Iterator;
import java.util.Vector;

import jp.co.sds_corp.ratemixer.fx.FxRate;
import jp.co.sds_corp.ratemixer.fx.Target;
import jp.co.sds_corp.transfer.ReceiveEvent;
import jp.co.sds_corp.transfer.filter.AbstractFilter;
import jp.co.sds_corp.util.config.IConfiguration;

/**
* 為替レートのフィルタリングを行なうオブジェクトの基底クラスです。
*/
public abstract class FxFilter extends AbstractFilter
{
	private static final long serialVersionUID = 1L;
	Vector<Target>	vecTgt = new Vector<Target>();

	/**
	* 指定された初期情報で初期化します。
	* @param	config	初期情報
	*/
	public void init( IConfiguration config ) throws Exception {
		super.init( config );
		Iterator<IConfiguration> itr = config.getChilds( "target" );
		while ( itr.hasNext()) {
			IConfiguration c = (IConfiguration )itr.next();
			Target tgt = new Target();
			tgt.init( c );
			vecTgt.add( tgt );
		}
	}

	/**
	* レートが監視対象であるかチェックする。
	* @param	ev		レート変更イベント
	*/
	public boolean isTarget( ReceiveEvent ev ) {
		if ( vecTgt.size() == 0 ) {
			return true;
		}

		boolean bRet = false;
		FxRate rate = (FxRate )ev.getData();
		for ( int i = 0; i < vecTgt.size(); i++ ) {
			Target tgt = (Target )vecTgt.get( i );
			if ( tgt.match( rate )) {
				if ( tgt.isExclude()) {
					return false;
				} else {
					bRet = true;
				}
			}
		}
		return bRet;
	}

	/**
	* レートのフィルタリングを行なう
	* @param	ev		レート変更イベント
	*/
	public abstract boolean isValid( ReceiveEvent ev ) throws Exception;
}
