package	jp.co.sds_corp.ratemixer.fx.mixer;

import jp.co.sds_corp.fxutil.CcyPair;
import jp.co.sds_corp.fxutil.IRateLoader;
import jp.co.sds_corp.fxutil.MasterCache;
import jp.co.sds_corp.ratemixer.fx.FxRate;
import jp.co.sds_corp.ratemixer.fx.command.ICommandServer;
import jp.co.sds_corp.transfer.ReceiveEvent;
import jp.co.sds_corp.transfer.mixer.SimpleMixer;
import jp.co.sds_corp.util.config.IConfiguration;
import jp.co.sds_corp.util.log.CmnLogFactory;
import jp.co.sds_corp.util.log.CmnLogger;

public class ForexMixer extends SimpleMixer implements IRateChangeListener
{
	private static final long serialVersionUID = 1L;
	CmnLogger logger = CmnLogFactory.getInstance().getLogger(this);
	MasterCache master = MasterCache.getInstance();
	ICommandServer	command;
	RateManager		manager;
	IRateLoader		loader;

	/**
	* 指定された初期情報で初期化します。
	* @param	config	初期情報
	*/
	public void init( IConfiguration config ) throws Exception {
		super.init( config );
		manager = new RateManager();
		loader = manager.getLoader();
		manager.addRateChangeListener( this );

		IConfiguration cmdcfg = config.getChild( "Command" );
		if ( cmdcfg != null ) {
			command = createCommandServer( cmdcfg );
		}
	}

	protected ICommandServer createCommandServer( IConfiguration config )
		throws Exception
	{
		String cls = config.getAttributes().get( "class" );
		ICommandServer ret = (ICommandServer)Class.forName( cls ).newInstance();
		ret.init( config );
		ret.setRateLoader( loader );
		return ret;
	}

	public void start() {
		if ( command != null ) {
			command.start();
		}
	}

	/**
	* データに何らかの変更があったときに呼び出されます。
	* @param	ev		イベントソースおよびデータの変更内容を指定します。
	*/
	public void received( ReceiveEvent ev ) throws Exception {
		FxRate rate = (FxRate )ev.getData();
		CcyPair pair = rate.getCcyPair();
		String var;
		var = pair.getProperty( "Spread" );
		if ( var != null ) {
			rate.setProperty( "Spread", var );
		}
		var = pair.getProperty( "Skew" );
		if ( var != null ) {
			rate.setProperty( "Skew",  var );
		}
		var = pair.getProperty( "Bias" );
		if ( var != null ) {
			rate.setProperty( "Bias",  var );
		}
		var = pair.getProperty( "Status" );
		if ( var != null ) {
			rate.setProperty( "Status",  var );
		} else {
			rate.setProperty( "Status",  "0" );
		}
		var = pair.getProperty( "Adjust1" );
		if ( var != null ) {
			rate.setProperty( "Adjust1",  var );
		} else {
			rate.setProperty( "Adjust1",  "0" );
		}
		var = pair.getProperty( "Adjust2" );
		if ( var != null ) {
			rate.setProperty( "Adjust2",  var );
		} else {
			rate.setProperty( "Adjust2",  "0" );
		}

		manager.rateReceived( rate );
	}

	public void rateChanged( FxRate rate ) {
		ReceiveEvent ev = new ReceiveEvent( this, rate );
		try {
			fireReceive( ev );
		} catch ( Exception ex ) {
			logger.error( "error", ex );
		}
	}

}
